"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var React = require("react");
var PropTypes = require("prop-types");
var react_native_1 = require("react-native");
var PopoverGeometry_1 = require("./PopoverGeometry");
var debounce = require('lodash.debounce');
var styles = react_native_1.StyleSheet.create({
    container: __assign({}, react_native_1.StyleSheet.absoluteFillObject, { opacity: 0, backgroundColor: 'transparent' }),
    containerVisible: {
        opacity: 1,
    },
    background: __assign({}, react_native_1.StyleSheet.absoluteFillObject, { backgroundColor: 'rgba(0,0,0,0.5)' }),
    content: {
        flexDirection: 'column',
        position: 'absolute',
        backgroundColor: '#f2f2f2',
        padding: 8,
    },
    shadow: {
        shadowColor: 'black',
        shadowOffset: { width: 0, height: 2 },
        shadowRadius: 2,
        shadowOpacity: 0.8,
    },
    arrow: {
        position: 'absolute',
        borderTopColor: '#f2f2f2',
        borderRightColor: 'transparent',
        borderBottomColor: 'transparent',
        borderLeftColor: 'transparent',
    },
});
var _a = react_native_1.Dimensions.get('window'), SCREEN_HEIGHT = _a.height, SCREEN_WIDTH = _a.width;
var ArrowRotation = {
    bottom: 2,
    left: -1,
    right: 1,
    top: 0.0,
    auto: 0.0,
};
var Popover = (function (_super) {
    __extends(Popover, _super);
    function Popover(props) {
        var _this = _super.call(this, props) || this;
        _this.updateState = debounce(_this.setState, 100);
        _this.measureContent = function (_a) {
            var _b = _a.nativeEvent.layout, width = _b.width, height = _b.height;
            if (width && height) {
                var contentSize = { width: width, height: height };
                var geom = PopoverGeometry_1.computeGeometry(contentSize, _this.props.placement, _this.props.fromRect, _this.props.displayArea, _this.props.arrowSize);
                var isAwaitingShow_1 = _this.state.isAwaitingShow;
                _this.updateState((__assign({}, geom, { contentSize: contentSize })), function () {
                    if (isAwaitingShow_1) {
                        _this.startAnimation(true);
                    }
                });
            }
        };
        _this.getTranslateOrigin = function () {
            var _a = _this.state, contentSize = _a.contentSize, origin = _a.origin, anchor = _a.anchor;
            var popoverCenter = { x: origin.x + contentSize.width / 2, y: origin.y + contentSize.height / 2 };
            return { x: anchor.x - popoverCenter.x, y: anchor.y - popoverCenter.y };
        };
        _this.startAnimation = function (show) {
            var values = _this.state.animations;
            var translateOrigin = _this.getTranslateOrigin();
            if (show) {
                values.translate.setValue(translateOrigin);
                values.rotate.setValue(ArrowRotation[_this.state.placement]);
            }
            var commonConfig = {
                duration: 300,
                easing: show ? react_native_1.Easing.out(react_native_1.Easing.back(1.70158)) : react_native_1.Easing.inOut(react_native_1.Easing.quad),
                useNativeDriver: true,
            };
            var doneCallback = show ? undefined : function () { return _this.setState({ visible: false }); };
            react_native_1.Animated.parallel([
                react_native_1.Animated.timing(values.rotate, {
                    toValue: ArrowRotation[_this.state.placement],
                    duration: 0,
                    useNativeDriver: true,
                }),
                react_native_1.Animated.timing(values.fade, __assign({ toValue: show ? 1 : 0 }, commonConfig)),
                react_native_1.Animated.timing(values.translate, __assign({ toValue: show ? { x: 0, y: 0 } : translateOrigin }, commonConfig)),
                react_native_1.Animated.timing(values.scale, __assign({ toValue: show ? 1 : 0 }, commonConfig)),
            ]).start(doneCallback);
        };
        _this.computeStyles = function () {
            var _a = _this.state, animations = _a.animations, anchor = _a.anchor, origin = _a.origin;
            var arrowSize = _this.props.arrowSize;
            var width = arrowSize.width + 2;
            var height = arrowSize.height * 2 + 2;
            return {
                background: [
                    styles.background,
                    _this.props.backgroundStyle,
                    {
                        opacity: animations.fade.interpolate({
                            inputRange: [0, 1],
                            outputRange: [0, 1],
                            extrapolate: 'clamp',
                        }),
                    },
                ],
                arrow: [
                    styles.arrow,
                    _this.props.arrowStyle,
                    {
                        left: anchor.x - origin.x - width / 2,
                        top: anchor.y - origin.y - height / 2,
                        width: width,
                        height: height,
                        borderTopWidth: height / 2,
                        borderRightWidth: width / 2,
                        borderBottomWidth: height / 2,
                        borderLeftWidth: width / 2,
                        transform: [
                            {
                                rotate: animations.rotate.interpolate({
                                    inputRange: [-2, 2],
                                    outputRange: ['-180deg', '180deg'],
                                    extrapolate: 'clamp',
                                }),
                            },
                            {
                                scale: animations.scale.interpolate({
                                    inputRange: [0, 1],
                                    outputRange: [0, 1],
                                    extrapolate: 'clamp',
                                }),
                            },
                        ],
                    },
                ],
                popover: [
                    _this.props.popoverStyle,
                ],
                content: [
                    styles.shadow,
                    styles.content,
                    _this.props.contentStyle,
                    {
                        transform: [
                            { translateX: animations.translate.x },
                            { translateY: animations.translate.y },
                            { scale: animations.scale },
                        ],
                    },
                ],
            };
        };
        _this.state = {
            contentSize: { width: 0, height: 0 },
            anchor: { x: 0, y: 0 },
            origin: { x: 0, y: 0 },
            placement: props.placement || 'auto',
            visible: false,
            isAwaitingShow: false,
            animations: {
                scale: new react_native_1.Animated.Value(0),
                translate: new react_native_1.Animated.ValueXY({ x: 0, y: 0 }),
                fade: new react_native_1.Animated.Value(0),
                rotate: new react_native_1.Animated.Value(0),
            },
        };
        return _this;
    }
    Popover.prototype.componentWillReceiveProps = function (nextProps) {
        var _this = this;
        var willBeVisible = nextProps.visible;
        var _a = this.props, visible = _a.visible, fromRect = _a.fromRect, displayArea = _a.displayArea;
        if (willBeVisible !== visible) {
            if (willBeVisible) {
                this.setState({ contentSize: { width: 0, height: 0 }, isAwaitingShow: true, visible: true });
            }
            else {
                this.startAnimation(false);
            }
        }
        else if (willBeVisible && (fromRect !== nextProps.fromRect || displayArea !== nextProps.displayArea)) {
            var contentSize = this.state.contentSize;
            var geom = PopoverGeometry_1.computeGeometry(contentSize, nextProps.placement, nextProps.fromRect, nextProps.displayArea, nextProps.arrowSize);
            var isAwaitingShow_2 = this.state.isAwaitingShow;
            this.setState(__assign({}, geom, { contentSize: contentSize }), function () {
                if (isAwaitingShow_2) {
                    _this.startAnimation(true);
                }
            });
        }
    };
    Popover.prototype.render = function () {
        var origin = this.state.origin;
        var computedStyles = this.computeStyles();
        var contentSizeAvailable = this.state.contentSize.width;
        return (<react_native_1.Modal transparent visible={this.state.visible} onRequestClose={this.props.onClose}>
        <react_native_1.View style={[styles.container, contentSizeAvailable && styles.containerVisible]}>

          <react_native_1.TouchableWithoutFeedback onPress={this.props.onClose}>
            <react_native_1.Animated.View style={computedStyles.background}/>
          </react_native_1.TouchableWithoutFeedback>

          <react_native_1.Animated.View style={[{ top: origin.y, left: origin.x }, computedStyles.popover]}>
            <react_native_1.Animated.View onLayout={this.measureContent} style={computedStyles.content}>
              {this.props.children}
            </react_native_1.Animated.View>
            <react_native_1.Animated.View style={computedStyles.arrow}/>
          </react_native_1.Animated.View>

        </react_native_1.View>
      </react_native_1.Modal>);
    };
    return Popover;
}(React.Component));
Popover.propTypes = {
    visible: PropTypes.bool,
    onClose: PropTypes.func,
    arrowSize: PropTypes.shape({
        x: PropTypes.number,
        y: PropTypes.number,
    }),
    placement: PropTypes.oneOf(['left', 'top', 'right', 'bottom', 'auto']),
    fromRect: PropTypes.shape({
        x: PropTypes.number,
        y: PropTypes.number,
        width: PropTypes.number,
        height: PropTypes.number,
    }),
    displayArea: PropTypes.shape({
        x: PropTypes.number,
        y: PropTypes.number,
        width: PropTypes.number,
        height: PropTypes.number,
    }),
    backgroundStyle: PropTypes.any,
    arrowStyle: PropTypes.any,
    popoverStyle: PropTypes.any,
    contentStyle: PropTypes.any,
};
Popover.defaultProps = {
    visible: false,
    onClose: function () {
    },
    displayArea: { x: 10, y: 10, width: SCREEN_WIDTH - 20, height: SCREEN_HEIGHT - 20 },
    arrowSize: { width: 16, height: 8 },
    placement: 'auto',
};
Popover.displayName = 'Popover';
exports.default = Popover;
//# sourceMappingURL=Popover.js.map